unit Graphker;

interface

uses windows, SysUtils, Graphics, Classes, Printers,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs,
   jmmutil, S250cfg, math;

const
   IntSansSignif   = -1;
   NbreCourbeMax   = 8;
   CouleurCalc     = clRed;
   CouleurTitre    = clBlack;
   dimCurseur      = 8;
   dimPoint        = 4;
   PrecisionTemps  = 7;
   Precision250    = 4;
   LongNombreAxe   = 4;
   NbreGraduations = 10;
   NbreGradMin     = 5;
   CouleurAxe: array[boolean] of Tcolor = (clGray, clBlack);
   CouleurRepere: Tcolor = clRed;
   CouleurCourbe: array[1..NbreCourbeMax] of Tcolor =
      (clBlue, clRed, clGreen, clPurple, clTeal, clNavy, clOlive, clBlack);

type
   TEtatSpectre   = (esBlancEnCours, esSpectreEnCours, esAttente);
   TEtatCinetique = (eaEnCours, eaTerminee, eaAttente);
   Ttrace         = (tLigne, tSpectre, tPoint);
   indiceCourbe   = 1..NbreCourbeMax;
   Enombre        = class(Exception);

   Trepere = class
      Valeur: double;
      Text:   string;
   end;

   Tgraphe = class;

   Tmonde = class
      mini, maxi: double;
      a, b:       double; { entier = rel*a+b }
      zeroInclus: boolean;
      avecAxe, grandAxe: boolean;
      nom:        string;
      unite:      string;
      signif:     String;
      PosCurseur: integer;
      deltaAxe, pointDebut: double;
      { deltaAxe = cart entre graduations sur les axes }
      constructor Create;
      procedure setDelta;
      procedure affecteEntier(r: double; var i: integer);
      function FormatAxe(x: double): string;
   end;

   Tgraphe = class
      codeErreur:     integer;
      dimPoint:       integer;
      TitreGr:        string;
      nbrePoints:     indiceVecteur;
      AbsRef:         vecteur;
      MinRef, MaxRef: double;
      mondeX, mondeY: Tmonde;
      LimiteCourbe, LimiteFenetre: Trect;
      LargCarac, HautCarac, Marge: integer;
      quadrillage:    boolean;
      canvas:         Tcanvas;
      XReticule, YReticule: integer;
      NbreCourbe:     integer;
      ListeRepere:    TList;
      valeurX:        vecteur;
      valeurY:        array[indiceCourbe] of vecteur;
      Titre:          array[indiceCourbe] of string;
      Trace:          Ttrace;
      Impression:     boolean;
      constructor Create;
      destructor Destroy; override;
      procedure Draw;
      procedure TraceLigne;
      procedure TracePoints;
      procedure TraceSpectre(c: indiceCourbe);
      procedure TraceRepere(repere: Trepere);
      procedure AjouteRepere(V: double; const T: string);
      procedure Reset;
      procedure WindowXY(xr, yr: double; var xi, yi: integer);
      procedure MondeXY(xi, yi: integer; var xr, yr: double);
      procedure ChercheMonde;
      procedure SetMonde(minX, minY, maxX, maxY: double);
      procedure DrawAxis;
      procedure TraceRef;
      procedure TraceFiltres;
      procedure TraceUnPoint(i: indiceVecteur; c: indiceCourbe);
      procedure SetHauteurFonte(H: integer);
      procedure TraceReticule;
      procedure TraceCurseur;
      procedure TraceCroix(x, y, largeur: integer);
      procedure TracePointBeer(x, y: double);
      procedure ModifCurseur(x, y: double);
      function NbreSpectre: integer;
      function MaxiY(c: indiceCourbe): double;
      function MaxiToutY : double;      
      function MaxiX: double;
      procedure Imprime;
   end;

procedure MyRectangle(Acanvas: Tcanvas; Arect: Trect);
procedure MyLine(Acanvas: Tcanvas; Arect: Trect);

implementation

constructor Tmonde.Create;
begin
   inherited Create;
   A := 1;
   B := 0;
   nom := '';
   unite := '';
   signif := '';
   ZeroInclus := True;
   Maxi := 255;
   Mini := 0;
   PosCurseur := intSansSignif;
end;{--- Tmonde.Create ---}

procedure Tmonde.affecteEntier(r: double; var i: integer);
begin
   if isnan(r) then
      raise Enombre.Create('Valeur non initialise')
   else
      i := round(A * r + B);
end;

procedure TGraphe.MondeXY(xi, yi: integer; var xr, yr: double);
begin
   xr := (xi - mondeX.B) / mondeX.A;
   yr := (yi - mondeY.B) / mondeY.A;
end;

procedure Tgraphe.SetHauteurFonte(H: integer);
begin
   with canvas.Font do
   begin
      if H <= 13 then
         H := 13 { MS sans serif 8 points = 13 pixels }
      else if H <= 18 then
         H := 16 { MS sans serif 10 points = 16 pixels }
      else if H <= 24 then
         H := 20; { MS sans serif 12 points = 20 pixels }
      if H <= 20 then
         Name := 'Regressi'
      else
         Name := 'Arial Regressi';
      Height := H;
      HautCarac := H;
      Marge := hautCarac div 3;
      LargCarac := 2 * H div 3;
   end;
end;

procedure TGraphe.WindowXY(xr, yr: double; var xi, yi: integer);
begin
   try
      mondeX.affecteEntier(xr, xi);
      mondeY.affecteEntier(yr, yi);
   except
      xi := IntSansSignif;
      yi := IntSansSignif
   end;
end;

procedure TGraphe.ChercheMonde;

   procedure chercheProjectionY;
   begin
      with mondeY do
      begin
         A := (limiteCourbe.Top - limiteCourbe.Bottom) / (Maxi - Mini);
         B := limiteCourbe.Bottom - Mini * A;
         setDelta;
      end;
   end; { chercheProjectionY }

   procedure chercheProjectionX;
   begin
      with mondeX do
      begin
         A := (limiteCourbe.right - limiteCourbe.left) / (Maxi - Mini);
         B := limiteCourbe.left - Mini * A;
         setDelta;
      end;
   end;

begin { ChercheMonde }
   codeErreur := 0;
   with limiteFenetre do
   begin
      mondeY.avecAxe := (bottom - top) > NbreGradMin * hautCarac * 5 div 4;
      mondeX.avecAxe := (right - left) > NbreGradMin * 4 * largCarac;
      mondeY.grandAxe := (bottom - top) > NbreGraduations * hautCarac * 5 div 4;
      mondeX.grandAxe := (right - left) > NbreGraduations * 4 * largCarac;
   end;
   limiteCourbe := limiteFenetre;
   Inc(limiteCourbe.left, marge);
   Inc(limiteCourbe.top, marge);
   Dec(limiteCourbe.bottom, marge);
   Dec(limiteCourbe.right, marge);
   if mondeY.avecAxe then
   begin
      Inc(limiteCourbe.left, LongNombreAxe * largCarac);
      Inc(limiteCourbe.top, hautCarac);
   end;
   if mondeX.avecAxe then
      Dec(limiteCourbe.bottom, hautCarac);
   chercheProjectionX;
   chercheProjectionY;
end; { ChercheMonde }

procedure TGraphe.SetMonde(minX, minY, maxX, maxY: double);
begin
   mondeX.mini := minx;
   mondeX.maxi := maxx;
   mondeY.mini := miny;
   mondeY.maxi := maxy;
end; { SetMonde }

procedure Tgraphe.TraceUnPoint(i: indiceVecteur; c: indiceCourbe);
var
   xi, yi, x0, y0, j: integer;
begin
   canvas.pen.color := CouleurCourbe[c];
   canvas.pen.style := psSolid;
   WindowXY(valeurX[i], valeurY[c, i], xi, yi);
   if (xi = IntSansSignif) or (yi = IntSansSignif) then
      exit;
   j := 0;
   case trace of
      tSpectre:
      begin
         if i = 0 then
            exit;
         for j := pred(i) downto S250.miniLambda do
            if valeurX[j] > 0 then
               break;
         if j <= S250.miniLambda then
            exit;
      end;
      tLigne:
      begin
         if i = 0 then
            exit;
         j := pred(i);
      end;
      tPoint:
      begin
         traceCroix(xi, yi, dimPoint);
         exit;
      end;
   end;
   WindowXY(valeurX[j], valeurY[c, j], x0, y0);
   if (x0 <> IntSansSignif) and (y0 <> IntSansSignif) then
      with canvas do
      begin
         moveTo(x0, y0);
         lineTo(xi, yi);
      end;
end;{traceUnPoint}

procedure Tgraphe.TraceRef;
var
   xi, yi: integer;
begin
   if minRef < maxRef then
      with canvas do
      begin
         pen.color := clBlue;
         pen.style := psSolid;
         pen.Width := 3;
         WindowXY(minRef, 0, xi, yi);
         moveTo(xi, yi);
         WindowXY(maxRef, 0, xi, yi);
         lineTo(xi, yi);
         pen.Width := 1;
      end;
end;{traceRef}

procedure Tgraphe.TraceFiltres;
var
   xi, yi: integer;
begin
   with canvas do
   begin
      pen.color := clRed;
      pen.style := psSolid;
      pen.Width := 3;
      WindowXY(S250.LambdaFiltreIR, 0, xi, yi);
      moveTo(xi, yi - 5);
      lineTo(xi, yi + 5);
      WindowXY(S250.LambdaFiltreUV, 0, xi, yi);
      moveTo(xi, yi - 5);
      lineTo(xi, yi + 5);
      pen.Width := 1;
   end;
end;{traceFiltres}

procedure TGraphe.TraceLigne;
var
   i: indiceVecteur;
   PointPrecedent: boolean;
   xi, yi: integer;
begin
   with canvas do
   begin
      traceRef;
      if NbrePoints = 0 then
         exit;
      pen.style := psSolid;
      pen.color := couleurCourbe[1];
      PointPrecedent := False;
      for i := 0 to pred(NbrePoints) do
      begin
         WindowXY(valeurX[i], valeurY[1, i], xi, yi);
         if (xi = IntSansSignif) or (yi = IntSansSignif) then
            pointPrecedent := False
         else if PointPrecedent then
            LineTo(xi, yi)
         else
         begin
            MoveTo(xi, yi);
            pointPrecedent := True;
         end;
      end;{for i}
   end;
end;{ TraceLigne }

procedure TGraphe.TracePoints;
var
   i: indiceVecteur;
   xi, yi: integer;
begin
   with canvas do
   begin
      if NbrePoints = 0 then
         exit;
      pen.style := psSolid;
      pen.color := couleurCourbe[1];
      for i := 0 to pred(NbrePoints) do
      begin
         WindowXY(valeurX[i], valeurY[1, i], xi, yi);
         if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
            TraceCroix(xi, yi, dimPoint);
      end;
   end;
end;{ TracePoints }

procedure TGraphe.TraceSpectre(c: indiceCourbe);
var
   i: indiceVecteur;
   PointPrecedent: boolean;
   xi, yi: integer;
begin
   with canvas do
   begin
      traceRef;
      pen.style := psSolid;
      pen.color := couleurCourbe[c];
      PointPrecedent := False;
      for i := 0 to maxVecteur do
         if valeurX[i] <> 0 then
         begin
            WindowXY(valeurX[i], valeurY[c, i], xi, yi);
            if (xi = IntSansSignif) or (yi = IntSansSignif) then
               pointPrecedent := False
            else if PointPrecedent then
               LineTo(xi, yi)
            else
            begin
               MoveTo(xi, yi);
               pointPrecedent := True;
            end;
         end;{for i}
   end;
end;{ TraceSpectre }

function TGraphe.NbreSpectre: integer;
var
   i: indiceVecteur;
begin
   Result := 0;
   for i := 0 to maxVecteur do
      if valeurX[i] <> 0 then
         Inc(Result);
end;

procedure TGraphe.Draw;
var
   c: integer;
begin
   with canvas do
   begin
      with limiteFenetre do
      begin
         dimPoint := (bottom - top) div 100;
         if dimPoint = 0 then
            dimPoint := 1;
         setHauteurFonte((bottom - top) div 30);
      end;
      pen.style := psDot;
      pen.color := CouleurAxe[Impression];
      brush.style := bsSolid;
      brush.color := clWhite;
      FillRect(LimiteFenetre);
      chercheMonde;
      drawAxis;
      case Trace of
         tSpectre: for c := 1 to NbreCourbe do
               traceSpectre(c);
         tLigne: traceLigne;
         tPoint: tracePoints;
      end;
      for c := 0 to pred(ListeRepere.Count) do
         TraceRepere(ListeRepere[c]);
   end;
end;

procedure TGraphe.Reset;
begin
   NbrePoints := 0;
   ListeRepere.Clear;
   NbreCourbe := 1;
end;

constructor TGraphe.Create;
var
   i: integer;
begin
   inherited Create;
   titreGr := '';
   impression := False;
   mondeX  := Tmonde.Create;
   mondeY  := Tmonde.Create;
   ListeRepere := TList.Create;
   quadrillage := True;
   NbrePoints := 0;
   for i := 0 to maxVecteur do
      AbsRef[i] := 0;
   NbreCourbe := 0;
   Trace := tLigne;
end;

destructor TGraphe.Destroy;
begin
   mondeX.Free;
   mondeY.Free;
   ListeRepere.Free;
   inherited Destroy;
end;

procedure MyRectangle(Acanvas: Tcanvas; Arect: Trect);
begin
   with Arect do
      Acanvas.Polygon([point(left, top), point(left, bottom), point(
         right, bottom), point(right, top)]);
end;

procedure MyLine(Acanvas: Tcanvas; Arect: Trect);
begin
   with Arect do
   begin
      Acanvas.MoveTo(left, top);
      Acanvas.LineTo(right, bottom);
   end;
end;

procedure Tmonde.SetDelta;
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const
   coeff: array[0..5] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
   largeur: double;
   Exponent: integer;
   index: byte;
begin
   Exponent := floor(Log10(Maxi - Mini));
   largeur := (maxi - mini) * power(10,-exponent);
   index := 0;
   if grandAxe then
      while ((largeur / coeff[index]) >= NbreGraduations) and (index < 5) do
         Inc(index)
   else
      while ((largeur / coeff[index]) >= NbreGradMin) and (index < 5) do
         Inc(index);
   DeltaAxe := coeff[index] * power(10,exponent);
   PointDebut := ceil(mini / deltaAxe) * deltaAxe;
end;

function Tmonde.FormatAxe(x: double): string;
begin
   FormatAxe := FloatToStrF(x, ffGeneral, 3, 0);
end;

procedure Tgraphe.TraceReticule;
begin
   with canvas do
   begin
      Pen.mode  := pmNotXor;
      Pen.color := clBlack;
      MoveTo(xReticule, clipRect.top);
      LineTo(xReticule, clipRect.bottom);
      MoveTo(clipRect.left, yReticule);
      LineTo(clipRect.right, yReticule);
      Pen.mode := pmCopy;
   end;
end;

procedure Tgraphe.TraceRepere(repere: Trepere);
var
   x, y: integer;
begin
   with Canvas, repere do
   begin
      Pen.color := couleurRepere;
      Pen.Style := psDash;
      windowXY(valeur, 0, x, y);
      MoveTo(x, LimiteCourbe.top);
      LineTo(x, LimiteCourbe.bottom);
      Pen.Style := psSolid;
   end;
end;{ TraceRepere }

procedure Tgraphe.AjouteRepere(V: double; const T: string);
var
   repere: Trepere;
begin
   repere := Trepere.Create;
   repere.valeur := V;
   repere.Text := T;
   ListeRepere.Add(repere);
   TraceRepere(repere);
end;

procedure Tgraphe.TraceCurseur;
begin
   if mondeX.posCurseur > intSansSignif then
      with canvas do
      begin
         Pen.Width := 3;
         Pen.color := couleurCourbe[1];
         Pen.Mode  := pmXor;
         TraceCroix(mondeX.posCurseur, mondeY.posCurseur, dimCurseur);
         Pen.Width := 1;
         Pen.Mode  := pmCopy;
      end;
end;{ TraceCurseur }

procedure Tgraphe.ModifCurseur(x, y: double);
begin
   TraceCurseur;
   windowXY(x, y, mondeX.posCurseur, mondeY.posCurseur);
   TraceCurseur;
end;{ ModifCurseur }

procedure Tgraphe.TracePointBeer(x, y: double);
var
   xi, yi: integer;
begin
   with canvas do
   begin
      Pen.color := couleurCalc;
      Brush.color := couleurCalc;
      Brush.style := bsSolid;
      WindowXY(x, y, xi, yi);
      FillRect(rect(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint));
   end;
end;{ TracePointBeer }

function Tgraphe.MaxiY(c: indiceCourbe): double;
var
   j: IndiceVecteur;
begin
   Result := 0;
   for j := 0 to maxVecteur do
      if valeurX[j] <> 0 then
         if valeurY[c, j] > Result then
            Result := valeurY[c, j];
end;{maxiY}

function Tgraphe.MaxiToutY : double;
var
   j: IndiceVecteur;
   c: indiceCourbe;
begin
   Result := 0;
   for c := 1 to NbreCourbe do begin
   for j := 0 to maxVecteur do
      if valeurX[j] <> 0 then
         if valeurY[c, j] > Result then
            Result := valeurY[c, j];
   end;
end;{maxiToutY}

function Tgraphe.MaxiX: double;
var
   j: IndiceVecteur;
begin
   Result := 0;
   for j := 0 to pred(NbrePoints) do
      if valeurX[j] > Result then
         Result := valeurX[j];
end;{maxiX}

procedure Tgraphe.TraceCroix(x, y, largeur: integer);
begin
   with canvas do
   begin
      moveTo(x, y - largeur);
      lineTo(x, y + largeur);
      moveTo(x - largeur, y);
      lineTo(x + largeur, y);
   end;
end;{ TraceCroix }

procedure Tgraphe.Imprime;
var
   sauveCanvas: Tcanvas;
begin
   if MessageDlg('Impression ?', mtConfirmation, mbOKCancel, 0) = mrOk then
   begin
      sauveCanvas := canvas;
      printer.Orientation := poLandscape;
      printer.beginDoc;
      canvas := printer.canvas;
      limiteFenetre := canvas.clipRect;
      impression := True;
      draw;
      impression := False;
      printer.endDoc;
      canvas := SauveCanvas;
   end;
end;

{$I axes}

end.

