unit Jmmutil;

interface

uses
   SysUtils, Windows, Messages, Classes, Dialogs, Forms,
   StdCtrls, Consts, Controls, shlObj;

const
   MaxVecteur = 1024;
   PrecisionSingle = 1e-7;
   PrecisionMax    = 7; { Precision double pour FloatToStrF }
   PrecisionMin    = 3;
   SymbReg: char   = '';
   SymbReg2: char  = '&';
   Tabulation      = #9;
   CaracCtrl       = '|';
   CodeCtrl: array[#0..#31] of string[3] =
      ('NUL', 'SOH', 'STX', 'ETX', 'EOT', 'ENQ', 'ACK', 'BEL', 'BS', 'TAB',
      'LF', 'VT', 'FF', 'CR', 'SO', 'SI', 'DLE', 'DC1', 'DC2', 'DC3',
      'DC4', 'NAK', 'SYN', 'ETB', 'CAN', 'EM', 'SUB', 'ESC', 'FS', 'GS',
      'RS', 'US');

type
   IndiceVecteur = 0..MaxVecteur;
   Vecteur       = array[IndiceVecteur] of double;
   CoefUnite = (zero, yocto, zepto, atto, femto, pico, nano, micro, milli,
      nulle, kilo, mega, giga, tera, peta, exa, zetta, yotta, infini);

const
   Majuscule: TsyscharSet = ['A'..'Z'];
   Minuscule: TsyscharSet = ['a'..'z'];
   Chiffre: TsyscharSet = ['0'..'9'];
   ChiffreEtc: TsyscharSet = ['0'..'9', '+', '-', 'e', 'E', '.'];
   ChiffreWin: TsyscharSet = ['0'..'9', '+', '-', 'e', 'E', '.', ','];
   crTab      = char(vk_tab);
   crSupprArr = char(vk_back);
   crHaut     = char(vk_up);
   crBas      = char(vk_down);
   crGauche   = char(vk_left);
   crDroite   = char(vk_right);
   crHome     = char(vk_home);
   crEnd      = char(vk_end);
   LettreLambda = WideChar($03BB);
   delta        = WideChar($03B4);
   crCR       = #13;
   crLF       = #10;
   crESC : ansiChar = #27;
   crAck      = #6;
   crBEL      = #7;
   crETX      = #3;
   crNoAck    = #21;
   separateur: TsysCharSet = [' ', crTab, ','];
   caracPrefixe: TsyscharSet =
      ['y', 'z', 'a', 'f', 'p', 'n', '', 'm', 'k', 'M', 'G', 'T', 'P', 'E'];

function ConvertitChaine(nombre: string; var valeur: double): boolean;
function FormatLong(nombre: double): string;
function FormatCourt(nombre: double): string;
function FloatToStrPoint(valeur: double): string;
function StrToFloatWin(nombre: string): double;
function EncodeDuree(duree: longint): TdateTime;
function DureeEcoulee(InstantDebut: TdateTime): longint;
procedure afficheErreur(texte: string);

procedure GetMinMax(const V: vecteur; N: indiceVecteur; var mini, maxi: double);
procedure VerifMinMaxReal(var mini, maxi: double);

var
   MesDocsDir, appDataDir: string;

implementation

function StrToFloatWin(nombre: String): double;
var
   posSeparator: byte;
begin
   if decimalSeparator <> '.' then begin
      posSeparator := pos('.', nombre);
      if posSeparator > 0 then
         Nombre[posSeparator] := decimalSeparator;
   end
   else if decimalSeparator <> ',' then begin
      posSeparator := pos(',', nombre);
      if posSeparator > 0 then
         Nombre[posSeparator] := decimalSeparator;
   end;
   StrToFloatWin := strToFloat(nombre);
end;

function FloatToStrPoint(valeur: double): string;
var
   nombre: string;
   posSeparator: byte;
begin
   nombre := FloatToStrF(valeur, ffGeneral, 7, 2);
   if decimalSeparator <> '.' then begin
      posSeparator := pos(decimalSeparator, nombre);
      if posSeparator > 0 then
         Nombre[posSeparator] := '.';
   end;
   Result := nombre;
end;

function ConvertitChaine(nombre: string; var valeur: double): boolean;
begin
   if (upcase(nombre[1]) = 'E') then
      nombre := '1' + nombre
   else if charinset(nombre[1],['.',',']) then
      nombre := '0' + nombre;
   convertitChaine := True;
   try
      valeur := StrToFloatWin(nombre);
   except
      convertitChaine := False;
   end;
end;

function FormatCourt(Nombre: double): string;
var
   Nexp: integer;
begin
   if abs(Nombre) < 1e-10 then
      Nexp := 2
   else if abs(Nombre) < 1e+10 then
      Nexp := 1
   else
      Nexp := 2;
   FormatCourt := FloatToStrF(nombre, ffGeneral, PrecisionMin, Nexp);
end;

function FormatLong(Nombre: double): string;
var
   Nexp: integer;
begin
   if abs(Nombre) < 1e-10 then
      Nexp := 2
   else if abs(Nombre) < 1e+10 then
      Nexp := 1
   else
      Nexp := 2;
   FormatLong := FloatToStrF(nombre, ffGeneral, PrecisionMax, Nexp);
end;

function EncodeDuree(duree: longint): TdateTime;
var
   h, m, s, ms: word;
begin
   h  := duree div 3600;
   duree := duree - h * 3600;
   m  := duree div 60;
   duree := duree - m * 60;
   s  := duree;
   ms := 0;
   EncodeDuree := EncodeTime(h, m, s, ms);
end;

function DureeEcoulee(InstantDebut: TdateTime): longint;
var
   h, m, s, ms: word;
begin
   decodeTime(time - instantDebut, h, m, s, ms);
   DureeEcoulee := h * 3600 + m * 60 + s;
end;

procedure afficheErreur(texte: string);
var
   SauveCursor: Tcursor;
begin
   SauveCursor := Screen.cursor;
   Screen.cursor := crDefault;
   MessageDlg(texte, mtError, [mbOK], 0);
   Screen.cursor := SauveCursor;
end;

procedure GetMinMax(const V: vecteur; N: indiceVecteur; var mini, maxi: double);
var
   i: indiceVecteur;
begin
   Mini := V[0];
   Maxi := V[0];
   for i := 1 to pred(N) do
      if V[i] > Maxi then
         Maxi := V[i]
      else if V[i] < Mini then
         Mini := V[i];
end;

procedure VerifMinMaxReal(var mini, maxi: double);
var
   z: double;
begin
   if Maxi < Mini then begin
      z := Maxi;
      Maxi := Mini;
      Mini := z;
   end;
end;

function AddBackSlash(aPath: string): string;
begin
   Result := EmptyStr;
   if (aPath <> '') and (aPath[Length(aPath)] <> '\') then
      Result := aPath + '\';
end;

// permet de retrouver un repertoire d'aprs sa valeur CSIDL
function GetFolder(CSIDLValue: integer): string;
var
   IdList: PITEMIDLIST;
   Folder: array[0..MAX_PATH] of char;
begin
   if not Failed(SHGetSpecialFolderLocation(0, CSIDLValue, IdList)) then begin
      SHGetPathFromIDList(IdList, Folder);
      Result := AddBackSlash(Folder);
   end;
end;

initialization
   mesDocsDir := GetFolder(CSIDL_Personal)+'Regressi';
   if not DirectoryExists(MesDocsDir) then
      createDir(MesDocsDir);
   mesDocsDir := AddBackSlash(MesDocsDir);
   appDataDir := GetFolder(CSIDL_AppData)+'Regressi';
   if not DirectoryExists(appDataDir) then
       createDir(appDataDir);
   appDataDir := AddBackSlash(appDataDir);
end.

